@@ -1,7 +1,7 @@
 /*
 	buffer.c: output buffer
 
-	copyright 1997-2006 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1997-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Oliver Fromme
 
@@ -10,13 +10,15 @@
 	- dammed night coders;-)
 */
 
-#include "mpg123.h"
+#include "mpg123app.h"
 
 #ifndef NOXFERMEM
 
+#include "common.h"
 #include <errno.h>
+#include "debug.h"
 
-int outburst = MAXOUTBURST;
+int outburst = 32768;
 
 static int intflag = FALSE;
 static int usr1flag = FALSE;
@@ -43,16 +45,22 @@ void real_buffer_ignore_lowmem(void)
 		xfermem_putcmd(buffermem->fd[XF_WRITER], XF_CMD_WAKEUP);
 }
 
-void real_buffer_end(void)
+void real_buffer_end(int rude)
 {
 	if (!buffermem)
 		return;
-	xfermem_putcmd(buffermem->fd[XF_WRITER], XF_CMD_TERMINATE);
+	xfermem_putcmd(buffermem->fd[XF_WRITER], rude ? XF_CMD_ABORT : XF_CMD_TERMINATE);
 }
 
 void real_buffer_resync(void)
 {
-	buffer_sig(SIGINT, TRUE);
+	if(buffermem->justwait)
+	{
+		buffermem->wakeme[XF_WRITER] = TRUE;
+		xfermem_putcmd(buffermem->fd[XF_WRITER], XF_CMD_RESYNC);
+		xfermem_getcmd(buffermem->fd[XF_WRITER], TRUE);
+	}
+	else buffer_sig(SIGINT, TRUE);
 }
 
 void real_plain_buffer_resync(void)
@@ -67,12 +75,18 @@ void real_buffer_reset(void)
 
 void real_buffer_start(void)
 {
-	buffer_sig(SIGCONT, FALSE);
+	if(buffermem->justwait)
+	{
+		debug("ending buffer's waiting");
+		buffermem->justwait = FALSE;
+		xfermem_putcmd(buffermem->fd[XF_WRITER], XF_CMD_WAKEUP);
+	}
 }
 
-void real_buffer_stop(void)
+void real_buffer_stop()
 {
-	buffer_sig(SIGSTOP, FALSE);
+	buffermem->justwait = TRUE;
+	buffer_sig(SIGINT, TRUE);
 }
 
 extern int buffer_pid;
@@ -92,9 +106,9 @@ void buffer_sig(int signal, int block)
 	return;
 }
 
-void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
+void buffer_loop(audio_output_t *ao, sigset_t *oldsigset)
 {
-	int bytes;
+	int bytes, outbytes;
 	int my_fd = buffermem->fd[XF_READER];
 	txfermem *xf = buffermem;
 	int done = FALSE;
@@ -103,10 +117,33 @@ void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
 	catchsignal (SIGINT, catch_interrupt);
 	catchsignal (SIGUSR1, catch_usr1);
 	sigprocmask (SIG_SETMASK, oldsigset, NULL);
-	if (param.outmode == DECODE_AUDIO) {
-		if (audio_open(ai) < 0) {
-			perror("audio");
-			exit(1);
+
+	xfermem_putcmd(my_fd, XF_CMD_WAKEUP);
+
+	debug("audio output: waiting for cap requests");
+	/* wait for audio setup queries */
+	while(1)
+	{
+		int cmd;
+		cmd = xfermem_block(XF_READER, xf);
+		if(cmd == XF_CMD_AUDIOCAP)
+		{
+			ao->rate     = xf->rate;
+			ao->channels = xf->channels;
+			ao->format   = ao->get_formats(ao);
+			debug3("formats for %liHz/%ich: 0x%x", ao->rate, ao->channels, ao->format);
+			xf->format = ao->format;
+			xfermem_putcmd(my_fd, XF_CMD_AUDIOCAP);
+		}
+		else if(cmd == XF_CMD_WAKEUP)
+		{
+			debug("got wakeup... leaving config mode");
+			break;
+		}
+		else
+		{
+			error1("unexpected command %i", cmd);
+			return;
 		}
 	}
 
@@ -117,14 +154,25 @@ void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
 
 	for (;;) {
 		if (intflag) {
+			debug("handle intflag... flushing");
 			intflag = FALSE;
-			if (param.outmode == DECODE_AUDIO)
-				audio_queueflush (ai);
-			xf->readindex = xf->freeindex;
-			if (xf->wakeme[XF_WRITER])
-				xfermem_putcmd(my_fd, XF_CMD_WAKEUP);
+			if (param.outmode == DECODE_AUDIO) ao->flush(ao);
+			/* Either prepare for waiting or empty buffer now. */
+			if(!xf->justwait) xf->readindex = xf->freeindex;
+			else
+			{
+				int cmd;
+				debug("Prepare for waiting; draining command queue. (There's a lot of wakeup commands pending, usually.)");
+				do
+				{
+					cmd = xfermem_getcmd(my_fd, FALSE);
+					/* debug1("drain: %i",  cmd); */
+				} while(cmd > 0);
+			}
+			if(xf->wakeme[XF_WRITER]) xfermem_putcmd(my_fd, XF_CMD_WAKEUP);
 		}
 		if (usr1flag) {
+			debug("handling usr1flag");
 			usr1flag = FALSE;
 			/*   close and re-open in order to flush
 			 *   the device's internal buffer before
@@ -140,15 +188,12 @@ void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
 			 */
 			if (xf->wakeme[XF_WRITER])
 				xfermem_putcmd(my_fd, XF_CMD_WAKEUP);
-			if (param.outmode == DECODE_AUDIO) {
-				audio_close (ai);
-				ai->rate = xf->buf[0]; 
-				ai->channels = xf->buf[1]; 
-				ai->format = xf->buf[2];
-				if (audio_open(ai) < 0) {
-					perror("audio");
-					exit(1);
-				}
+			ao->rate = xf->rate; 
+			ao->channels = xf->channels; 
+			ao->format = xf->format;
+			if (reset_output(ao) < 0) {
+				error1("failed to reset audio: %s", strerror(errno));
+				exit(1);
 			}
 		}
 		if ( (bytes = xfermem_get_usedspace(xf)) < outburst ) {
@@ -160,19 +205,19 @@ void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
 			if(preload < outburst)
 				preload = outburst;
 		}
-		if(bytes < preload) {
+		debug1("bytes: %i", bytes);
+		if(xf->justwait || bytes < preload) {
 			int cmd;
 			if (done && !bytes) { 
 				break;
 			}
 			
-			if(!done) {
+			if(xf->justwait || !done) {
 
 				/* Don't spill into errno check below. */
 				errno = 0;
-
 				cmd = xfermem_block(XF_READER, xf);
-
+				debug1("got %i", cmd);
 				switch(cmd) {
 
 					/* More input pending. */
@@ -183,23 +228,34 @@ void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
 					 */
 					case XF_CMD_WAKEUP:
 						break;	/* Proceed playing. */
-					case XF_CMD_TERMINATE:
-						/* Proceed playing without 
-						 * blocking any further.
-						 */
-						done=TRUE;
+					case XF_CMD_ABORT: /* Immediate end, discard buffer contents. */
+						return; /* Cleanup happens outside of buffer_loop()*/
+					case XF_CMD_TERMINATE: /* Graceful end, playing stuff in buffer and then return. */
+						debug("going to terminate");
+						done = TRUE;
+						break;
+					case XF_CMD_RESYNC:
+						debug("ordered resync");
+						if (param.outmode == DECODE_AUDIO) ao->flush(ao);
+
+						xf->readindex = xf->freeindex;
+						if (xf->wakeme[XF_WRITER]) xfermem_putcmd(my_fd, XF_CMD_WAKEUP);
+						continue;
 						break;
 					case -1:
-						if(errno==EINTR)
+						if(intflag || usr1flag) /* Got signal, handle it at top of loop... */
+						{
+							debug("buffer interrupted");
 							continue;
+						}
 						if(errno)
-							perror("Yuck! Error in buffer handling...");
+							error1("Yuck! Error in buffer handling... or somewhere unexpected: %s", strerror(errno));
 						done = TRUE;
 						xf->readindex = xf->freeindex;
 						xfermem_putcmd(xf->fd[XF_READER], XF_CMD_TERMINATE);
 						break;
 					default:
-						fprintf(stderr, "\nEh!? Received unknown command 0x%x in buffer process. Tell Daniel!\n", cmd);
+						fprintf(stderr, "\nEh!? Received unknown command 0x%x in buffer process.\n", cmd);
 				}
 			}
 		}
@@ -207,7 +263,7 @@ void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
 		 * audio settings. We do not want to lower the preload mark
 		 * just yet!
 		 */
-		if (!bytes)
+		if (xf->justwait || !bytes)
 			continue;
 		preload = outburst; /* set preload to lower mark */
 		if (bytes > xf->size - xf->readindex)
@@ -215,16 +271,14 @@ void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
 		if (bytes > outburst)
 			bytes = outburst;
 
-		if (param.outmode == DECODE_FILE)
-			bytes = write(OutputDescriptor, xf->data + xf->readindex, bytes);
-		else if (param.outmode == DECODE_AUDIO)
-			bytes = audio_play_samples(ai,
-				(unsigned char *) (xf->data + xf->readindex), bytes);
+		debug("write");
+		outbytes = flush_output(ao, (unsigned char*) xf->data + xf->readindex, bytes);
 
-		if(bytes < 0) {
-			bytes = 0;
-			if(errno != EINTR) {
-				perror("Ouch ... error while writing audio data: ");
+		if(outbytes < bytes)
+		{
+			if(outbytes < 0) outbytes = 0;
+			if(!intflag && !usr1flag) {
+				error1("Ouch ... error while writing audio data: %s", strerror(errno));
 				/*
 				 * done==TRUE tells writer process to stop
 				 * sending data. There might be some latency
@@ -238,15 +292,14 @@ void buffer_loop(struct audio_info_struct *ai, sigset_t *oldsigset)
 				xf->readindex = xf->freeindex;
 				xfermem_putcmd(xf->fd[XF_READER], XF_CMD_TERMINATE);
 			}
+			else debug("buffer interrupted");
 		}
+		bytes = outbytes;
 
 		xf->readindex = (xf->readindex + bytes) % xf->size;
 		if (xf->wakeme[XF_WRITER])
 			xfermem_putcmd(my_fd, XF_CMD_WAKEUP);
 	}
-
-	if (param.outmode == DECODE_AUDIO)
-		audio_close (ai);
 }
 
 #endif
